@@ -10,25 +10,23 @@ use crate::file_chooser::FileChooser;
 use crate::filetypes::{CompressionType, FileType, OutputType};
 use crate::input_file::InputFile;
 use crate::magick::{
-    count_frames, generate_job, wait_for_child, GhostScriptConvertJob, JobFile, MagickConvertJob,
-    ResizeArgument,
+    JobFile, MagickConvertJob, ResizeArgument, count_frames, generate_job, wait_for_child,
 };
 use crate::temp::{clean_dir, create_temporary_dir, get_temp_file_path};
 use crate::widgets::about_window::SwitcherooAbout;
 use crate::widgets::image_rest::ImageRest;
 use crate::widgets::image_thumbnail::ImageThumbnail;
-use crate::{runtime, GHOST_SCRIPT_BINARY_NAME, ZIP_BINARY_NAME};
+use crate::{ZIP_BINARY_NAME, runtime};
 use adw::prelude::*;
 use futures::future::join_all;
 use gettextrs::gettext;
-use glib::{clone, idle_add_local_once, MainContext};
+use glib::{MainContext, clone, idle_add_local_once};
 use gtk::accessible::Property;
 use gtk::gdk::Texture;
 use gtk::{gdk, gio, glib, subclass::prelude::*};
 use itertools::Itertools;
 use shared_child::SharedChild;
 use std::sync::Arc;
-use tokio::spawn;
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum ResizeFilter {
@@ -170,6 +168,8 @@ mod imp {
         pub dpi_row: TemplateChild<adw::ActionRow>,
         #[template_child]
         pub navigation: TemplateChild<adw::NavigationView>,
+        #[template_child]
+        pub help_overlay: TemplateChild<adw::ShortcutsDialog>,
 
         pub provider: gtk::CssProvider,
         #[derivative(Default(value = "gio::ListStore::new::<InputFile>()"))]
@@ -243,7 +243,9 @@ mod imp {
 glib::wrapper! {
     pub struct AppWindow(ObjectSubclass<imp::AppWindow>)
         @extends gtk::Widget, gtk::Window,  gtk::ApplicationWindow, adw::ApplicationWindow,
-        @implements gio::ActionMap, gio::ActionGroup, gtk::Root;
+        @implements gio::ActionMap, gio::ActionGroup,
+                    gtk::Root, gtk::Native, gtk::ShortcutManager,
+                    gtk::Accessible, gtk::Buildable, gtk::ConstraintTarget;
 }
 
 #[gtk::template_callbacks]
@@ -293,6 +295,15 @@ impl AppWindow {
                     }
                 ))
                 .build(),
+            gio::ActionEntry::builder("show-help-overlay")
+                .activate(clone!(
+                    #[weak(rename_to=window)]
+                    self,
+                    move |_, _, _| {
+                        window.show_help_overlay();
+                    }
+                ))
+                .build(),
             gio::ActionEntry::builder("about")
                 .activate(clone!(
                     #[weak(rename_to=window)]
@@ -544,6 +555,10 @@ impl AppWindow {
         SwitcherooAbout::show(self);
     }
 
+    fn show_help_overlay(&self) {
+        self.imp().help_overlay.present(Some(self));
+    }
+
     fn close_dialog(&self) {
         let stop_converting_dialog = adw::AlertDialog::new(
             Some(&gettext("Stop converting?")),
@@ -583,7 +598,7 @@ impl AppWindow {
     }
 
     fn set_convert_progress(&self, done: usize, total: usize) {
-        let msg = format!("{}/{}", done, total);
+        let msg = format!("{done}/{total}");
         self.imp().progress_bar.set_text(Some(&msg));
         self.imp()
             .progress_bar
@@ -645,7 +660,6 @@ impl AppWindow {
         self.imp().removed.replace(HashSet::new());
 
         self.switch_to_stack_loading_generally();
-        // self.switch_to_stack_loading();
 
         for file in files.iter() {
             self.imp().input_file_store.append(file);
@@ -660,28 +674,24 @@ impl AppWindow {
         let files = self.files();
         let file_paths = files.iter().map(|f| f.path()).collect_vec();
 
-        let (sender, receiver) = async_channel::bounded(1);
+        let (sender, receiver) = tokio::sync::oneshot::channel();
 
         std::thread::spawn(move || {
-            let rt = tokio::runtime::Builder::new_multi_thread()
-                .enable_all()
-                .build()
-                .unwrap();
             let jobs = file_paths
                 .into_iter()
                 .map(|f| async move { count_frames(f).await.unwrap_or((1, None)) })
                 .collect_vec();
 
-            let res = rt.block_on(join_all(jobs));
+            let res = runtime().block_on(join_all(jobs));
 
-            sender.send_blocking(res).expect("Concurrency Issues");
+            sender.send(res).expect("Concurrency Issues");
         });
 
         glib::spawn_future_local(clone!(
             #[weak(rename_to=this)]
             self,
             async move {
-                while let Ok(image_info) = receiver.recv().await {
+                if let Ok(image_info) = receiver.await {
                     let real_files = files.clone();
                     for (f, (frame, dims)) in real_files.iter().zip(image_info.iter()) {
                         f.set_frames(*frame);
@@ -871,33 +881,26 @@ impl AppWindow {
 
         let (sender, receiver) = async_channel::bounded(1);
         std::thread::spawn(move || {
-            let rt = tokio::runtime::Builder::new_multi_thread()
-                .enable_all()
-                .build()
-                .unwrap();
             let file_paths_pixbuf = file_path_things
                 .into_iter()
                 .enumerate()
                 .map(|(i, (b, path))| {
                     let sender = sender.clone();
                     async move {
-                        spawn(async move {
-                            sender
-                                .send_blocking((
-                                    i,
-                                    match b {
-                                        true => Some(Texture::from_filename(&path)),
-                                        false => None,
-                                    },
-                                ))
-                                .expect("Concurrency Issues");
-                        })
-                        .await
+                        sender
+                            .send_blocking((
+                                i,
+                                match b {
+                                    true => Some(Texture::from_filename(&path)),
+                                    false => None,
+                                },
+                            ))
+                            .expect("Concurrency Issues");
                     }
                 })
                 .collect_vec();
 
-            rt.block_on(join_all(file_paths_pixbuf));
+            runtime().block_on(join_all(file_paths_pixbuf));
         });
 
         let completed = std::sync::Arc::new(AtomicUsize::new(0));
@@ -980,10 +983,9 @@ impl AppWindow {
                     (Pdf, _, c) => (0..c)
                         .map(|f| {
                             (
-                                path.clone(),
+                                format!("{path}[{f}]"),
                                 input_filetype,
                                 format!("{output_stem}[{f}].{}", output_type.as_extension()),
-                                f,
                             )
                         })
                         .collect_vec(),
@@ -991,7 +993,6 @@ impl AppWindow {
                         format!("{path}[0]"),
                         input_filetype,
                         format!("{output_stem}.{}", output_type.as_extension()),
-                        0,
                     )],
                     (input, output, _)
                         if input.supports_animation() && output.supports_animation() =>
@@ -1000,7 +1001,6 @@ impl AppWindow {
                             path,
                             input_filetype,
                             format!("{output_stem}.{}", output_type.as_extension()),
-                            0,
                         )]
                     }
                     (input, _, count) if input.supports_animation() => (0..count)
@@ -1009,7 +1009,6 @@ impl AppWindow {
                                 format!("{path}[{f}]"),
                                 input_filetype,
                                 format!("{output_stem}[{f}].{}", output_type.as_extension()),
-                                f,
                             )
                         })
                         .collect_vec(),
@@ -1017,7 +1016,6 @@ impl AppWindow {
                         format!("{path}[0]"),
                         input_filetype,
                         format!("{output_stem}.{}", output_type.as_extension()),
-                        0,
                     )],
                 }
             })
@@ -1027,7 +1025,7 @@ impl AppWindow {
 
         let output_files = job_input
             .iter()
-            .map(|(_, _, o, _)| {
+            .map(|(_, _, o)| {
                 get_temp_file_path(&dir, JobFile::new(output_type, Some(o.to_string())))
                     .to_str()
                     .unwrap()
@@ -1044,31 +1042,23 @@ impl AppWindow {
             quality: self.get_quality_argument(),
             filter: self.get_filter_argument(),
             resize_arg: self.get_resize_argument(),
-            coalesce: false,
+            density: None,
             first_frame: false,
             remove_alpha: false,
         };
 
-        let ghost_arguments = GhostScriptConvertJob {
-            input_file: "".to_string(),
-            output_file: "".to_string(),
-            page: 0,
-            dpi: self.get_dpi_argument(),
-        };
-
         let magick_jobs = job_input
             .into_iter()
-            .map(|(f, ft, os, frame)| {
+            .map(|(f, ft, os)| {
                 generate_job(
                     &f,
-                    frame,
                     &ft,
                     get_temp_file_path(&dir, JobFile::new(output_type, Some(os)))
                         .to_str()
                         .unwrap(),
                     &output_type,
-                    &dir,
-                    (&magick_arguments, &ghost_arguments),
+                    self.get_dpi_argument(),
+                    &magick_arguments,
                 )
             })
             .collect_vec();
@@ -1083,11 +1073,6 @@ impl AppWindow {
         let stop_flag_s = stop_flag.clone();
 
         std::thread::spawn(move || {
-            let rt = tokio::runtime::Builder::new_multi_thread()
-                .enable_all()
-                .build()
-                .unwrap();
-
             let stop_flag = stop_flag_s.clone();
 
             let jobs = magick_jobs
@@ -1096,64 +1081,47 @@ impl AppWindow {
                     let stop_flag = stop_flag.clone();
                     let sender = sender.clone();
                     async move {
-                        spawn(async move {
-                            for mut mj_command in mjs.into_iter().map(|mj| mj.get_command()) {
-                                if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
-                                    return;
-                                }
-
-                                let mj_command_str =
-                                    mj_command.get_program().to_str().unwrap().to_string();
-
-                                let shared_child = match SharedChild::spawn(&mut mj_command) {
-                                    std::io::Result::Ok(shared_child) => shared_child,
-                                    std::io::Result::Err(e) => {
-                                        if mj_command_str == GHOST_SCRIPT_BINARY_NAME
-                                            && e.kind() == std::io::ErrorKind::NotFound
-                                        {
-                                            sender
-                                                .send_blocking(ArcOrOptionError::OptionError(Some(
-                                                    gs_missing(),
-                                                )))
-                                                .expect("Concurrency Issues");
-                                            return;
-                                        }
-                                        sender
-                                            .send_blocking(ArcOrOptionError::OptionError(Some(
-                                                mj_command_str + ": " + &e.to_string(),
-                                            )))
-                                            .expect("Concurrency Issues");
-                                        return;
-                                    }
-                                };
-
-                                if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
-                                    return;
-                                }
-                                let child_arc = std::sync::Arc::new(shared_child);
-                                sender
-                                    .send_blocking(ArcOrOptionError::Child(child_arc.clone()))
-                                    .expect("Concurrency Issues");
-                                let output = match wait_for_child(child_arc.clone()).await {
-                                    Ok(_) => None,
-                                    Err(e) => Some(e),
-                                };
-                                if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
-                                    return;
-                                }
-
-                                sender
-                                    .send_blocking(ArcOrOptionError::OptionError(output))
-                                    .expect("Concurrency Issues");
+                        for mut mj_command in mjs.into_iter().map(|mj| mj.get_command()) {
+                            if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
+                                return;
                             }
-                        })
-                        .await
-                        .ok();
+
+                            let mj_command_str =
+                                mj_command.get_program().to_str().unwrap().to_string();
+
+                            let shared_child = match SharedChild::spawn(&mut mj_command) {
+                                std::io::Result::Ok(shared_child) => shared_child,
+                                std::io::Result::Err(e) => {
+                                    sender
+                                        .send_blocking(ArcOrOptionError::OptionError(Some(
+                                            mj_command_str + ": " + &e.to_string(),
+                                        )))
+                                        .expect("Concurrency Issues");
+                                    return;
+                                }
+                            };
+
+                            if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
+                                return;
+                            }
+                            let child_arc = std::sync::Arc::new(shared_child);
+                            sender
+                                .send_blocking(ArcOrOptionError::Child(child_arc.clone()))
+                                .expect("Concurrency Issues");
+                            let output = wait_for_child(child_arc.clone()).await.err();
+                            if stop_flag.load(std::sync::atomic::Ordering::SeqCst) {
+                                return;
+                            }
+
+                            sender
+                                .send_blocking(ArcOrOptionError::OptionError(output))
+                                .expect("Concurrency Issues");
+                        }
                     }
                 })
                 .collect_vec();
 
-            rt.block_on(join_all(jobs));
+            runtime().block_on(join_all(jobs));
         });
 
         let dir_path = dir.path().to_str().unwrap().to_string();
@@ -1272,23 +1240,6 @@ trait SettingsStore {
     fn load_selected_compression(&self) -> CompressionType;
 }
 
-fn does_binary_exist(binary: &str) -> bool {
-    std::process::Command::new("which")
-        .arg(binary)
-        .stdout(std::process::Stdio::null())
-        .status()
-        .map(|status| status.success())
-        .unwrap_or_default()
-}
-
-fn gs_missing() -> String {
-    gettext("The 'gs' (GhostScript) command is not available on your system. Please install GhostScript to convert multiple files to PDF.")
-}
-
-fn zip_missing() -> String {
-    gettext("The 'zip' command is not available on your system. Please install zip to convert multiple files to ZIP.")
-}
-
 impl ConvertOperations for AppWindow {
     fn convert_start_wrapper(&self, save_format: OutputType, path: String) {
         MainContext::default().spawn_local(clone!(
@@ -1317,28 +1268,26 @@ impl ConvertOperations for AppWindow {
         self.set_collecting_progress();
         let receiver = match save_format {
             OutputType::File(FileType::Pdf) if output_files.len() > 1 => {
-                if !does_binary_exist(GHOST_SCRIPT_BINARY_NAME) {
-                    self.convert_failed(gs_missing(), dir_path.clone());
-                    return;
-                }
-
                 let (sender, receiver) = async_channel::bounded(1);
 
                 std::thread::spawn(move || {
-                    let rt = tokio::runtime::Builder::new_multi_thread()
-                        .enable_all()
-                        .build()
-                        .unwrap();
-
-                    let shared_child: SharedChild = SharedChild::spawn(
-                        std::process::Command::new(GHOST_SCRIPT_BINARY_NAME)
-                            .arg("-dNOPAUSE")
-                            .arg("-sDEVICE=pdfwrite")
-                            .arg(format!("-sOUTPUTFILE={}", path))
-                            .arg("-dBATCH")
-                            .args(output_files),
-                    )
-                    .unwrap();
+                    let shared_child: SharedChild = match SharedChild::spawn(
+                        std::process::Command::new("magick")
+                            .stdout(std::process::Stdio::piped())
+                            .stderr(std::process::Stdio::piped())
+                            .args(output_files)
+                            .arg(path),
+                    ) {
+                        Ok(shared_child) => shared_child,
+                        Err(e) => {
+                            sender
+                                .send_blocking(ArcOrOptionError::OptionError(Some(
+                                    "magick: ".to_string() + &e.to_string(),
+                                )))
+                                .expect("Concurrency Issues");
+                            return;
+                        }
+                    };
                     let child_arc = std::sync::Arc::new(shared_child);
 
                     sender
@@ -1347,10 +1296,7 @@ impl ConvertOperations for AppWindow {
 
                     sender
                         .send_blocking(ArcOrOptionError::OptionError(
-                            match rt.block_on(wait_for_child(child_arc)) {
-                                Err(e) => Some(e),
-                                _ => None,
-                            },
+                            runtime().block_on(wait_for_child(child_arc)).err(),
                         ))
                         .expect("Concurrency Issues");
                 });
@@ -1363,14 +1309,14 @@ impl ConvertOperations for AppWindow {
                 let (sender, receiver) = async_channel::bounded(1);
 
                 std::thread::spawn(move || {
-                    let rt = tokio::runtime::Builder::new_multi_thread()
-                        .enable_all()
-                        .build()
-                        .unwrap();
-
-                    let shared_child: SharedChild =
-                        SharedChild::spawn(std::process::Command::new("mv").arg(file).arg(path))
-                            .unwrap();
+                    let shared_child: SharedChild = SharedChild::spawn(
+                        std::process::Command::new("mv")
+                            .stdout(std::process::Stdio::piped())
+                            .stderr(std::process::Stdio::piped())
+                            .arg(file)
+                            .arg(path),
+                    )
+                    .unwrap();
                     let child_arc = std::sync::Arc::new(shared_child);
 
                     sender
@@ -1379,10 +1325,7 @@ impl ConvertOperations for AppWindow {
 
                     sender
                         .send_blocking(ArcOrOptionError::OptionError(
-                            match rt.block_on(wait_for_child(child_arc)) {
-                                Err(e) => Some(e),
-                                _ => None,
-                            },
+                            runtime().block_on(wait_for_child(child_arc)).err(),
                         ))
                         .expect("Concurrency Issues");
                 });
@@ -1393,13 +1336,10 @@ impl ConvertOperations for AppWindow {
                 let (sender, receiver) = async_channel::bounded(1);
 
                 std::thread::spawn(move || {
-                    let rt = tokio::runtime::Builder::new_multi_thread()
-                        .enable_all()
-                        .build()
-                        .unwrap();
-
                     let shared_child: SharedChild = SharedChild::spawn(
                         std::process::Command::new("mv")
+                            .stdout(std::process::Stdio::piped())
+                            .stderr(std::process::Stdio::piped())
                             .args(output_files)
                             .arg(path),
                     )
@@ -1412,10 +1352,7 @@ impl ConvertOperations for AppWindow {
 
                     sender
                         .send_blocking(ArcOrOptionError::OptionError(
-                            match rt.block_on(wait_for_child(child_arc)) {
-                                Err(e) => Some(e),
-                                _ => None,
-                            },
+                            runtime().block_on(wait_for_child(child_arc)).err(),
                         ))
                         .expect("Concurrency Issues");
                 });
@@ -1425,24 +1362,26 @@ impl ConvertOperations for AppWindow {
             _ => {
                 let (sender, receiver) = async_channel::bounded(1);
 
-                if !does_binary_exist(ZIP_BINARY_NAME) {
-                    self.convert_failed(zip_missing(), dir_path.clone());
-                    return;
-                }
-
                 std::thread::spawn(move || {
-                    let rt = tokio::runtime::Builder::new_multi_thread()
-                        .enable_all()
-                        .build()
-                        .unwrap();
-
-                    let shared_child: SharedChild = SharedChild::spawn(
+                    let shared_child = match SharedChild::spawn(
                         std::process::Command::new(ZIP_BINARY_NAME)
                             .arg("-jFSm0")
                             .arg(path)
-                            .args(output_files),
-                    )
-                    .unwrap();
+                            .args(output_files)
+                            .stdout(std::process::Stdio::piped())
+                            .stderr(std::process::Stdio::piped()),
+                    ) {
+                        Ok(shared_child) => shared_child,
+                        Err(e) => {
+                            sender
+                                .send_blocking(ArcOrOptionError::OptionError(Some(
+                                    ZIP_BINARY_NAME.to_string() + ": " + &e.to_string(),
+                                )))
+                                .expect("Concurrency Issues");
+                            return;
+                        }
+                    };
+
                     let child_arc = std::sync::Arc::new(shared_child);
 
                     sender
@@ -1451,10 +1390,7 @@ impl ConvertOperations for AppWindow {
 
                     sender
                         .send_blocking(ArcOrOptionError::OptionError(
-                            match rt.block_on(wait_for_child(child_arc)) {
-                                Err(e) => Some(e),
-                                _ => None,
-                            },
+                            runtime().block_on(wait_for_child(child_arc)).err(),
                         ))
                         .expect("Concurrency Issues");
                 });
@@ -1811,78 +1747,75 @@ impl WindowUI for AppWindow {
             imp.resize_filter_row.set_visible(true);
         }
 
-        if input_filetypes
-            .iter()
-            .any(|input_filetype| *input_filetype == FileType::Pdf)
-        {
+        if input_filetypes.contains(&FileType::Pdf) {
             imp.dpi_row.set_visible(true);
         }
     }
 
     fn update_width_from_height(&self) {
-        if self.imp().link_axis.is_active() && self.imp().link_axis.is_visible() {
-            if let (Some(image_width), Some(image_height)) =
+        if self.imp().link_axis.is_active()
+            && self.imp().link_axis.is_visible()
+            && let (Some(image_width), Some(image_height)) =
                 (self.imp().image_width.get(), self.imp().image_height.get())
-            {
-                let old_value = self.imp().resize_width_value.text().as_str().to_owned();
-                let other_text = self.imp().resize_height_value.text().as_str().to_owned();
-                if other_text.is_empty() {
-                    return;
-                }
+        {
+            let old_value = self.imp().resize_width_value.text().as_str().to_owned();
+            let other_text = self.imp().resize_height_value.text().as_str().to_owned();
+            if other_text.is_empty() {
+                return;
+            }
 
-                let other_way = generate_height_from_width(
-                    old_value.parse().unwrap_or_default(),
-                    (image_width, image_height),
-                )
-                .to_string();
+            let other_way = generate_height_from_width(
+                old_value.parse().unwrap_or_default(),
+                (image_width, image_height),
+            )
+            .to_string();
 
-                if other_way == other_text {
-                    return;
-                }
+            if other_way == other_text {
+                return;
+            }
 
-                let new_value = generate_width_from_height(
-                    other_text.parse().unwrap_or_default(),
-                    (image_width, image_height),
-                )
-                .to_string();
+            let new_value = generate_width_from_height(
+                other_text.parse().unwrap_or_default(),
+                (image_width, image_height),
+            )
+            .to_string();
 
-                if old_value != new_value && new_value != "0" {
-                    self.imp().resize_width_value.set_text(&new_value);
-                }
+            if old_value != new_value && new_value != "0" {
+                self.imp().resize_width_value.set_text(&new_value);
             }
         }
     }
 
     fn update_height_from_width(&self) {
-        if self.imp().link_axis.is_active() && self.imp().link_axis.is_visible() {
-            if let (Some(image_width), Some(image_height)) =
+        if self.imp().link_axis.is_active()
+            && self.imp().link_axis.is_visible()
+            && let (Some(image_width), Some(image_height)) =
                 (self.imp().image_width.get(), self.imp().image_height.get())
-            {
-                let old_value = self.imp().resize_height_value.text().as_str().to_owned();
-                let other_text = self.imp().resize_width_value.text().as_str().to_owned();
-                if other_text.is_empty() {
-                    return;
-                }
+        {
+            let old_value = self.imp().resize_height_value.text().as_str().to_owned();
+            let other_text = self.imp().resize_width_value.text().as_str().to_owned();
+            if other_text.is_empty() {
+                return;
+            }
 
-                let other_way = generate_width_from_height(
-                    old_value.parse().unwrap_or_default(),
-                    (image_width, image_height),
-                )
-                .to_string();
+            let other_way = generate_width_from_height(
+                old_value.parse().unwrap_or_default(),
+                (image_width, image_height),
+            )
+            .to_string();
 
-                if other_way == other_text {
-                    return;
-                }
+            if other_way == other_text {
+                return;
+            }
 
-                let new_value = generate_height_from_width(
-                    other_text.parse().unwrap_or_default(),
-                    (image_width, image_height),
-                )
-                .to_string();
+            let new_value = generate_height_from_width(
+                other_text.parse().unwrap_or_default(),
+                (image_width, image_height),
+            )
+            .to_string();
 
-                if old_value != new_value && new_value != "0" {
-                    self.imp().resize_height_value.set_text(&new_value);
-                }
+            if old_value != new_value && new_value != "0" {
+                self.imp().resize_height_value.set_text(&new_value);
             }
         }
     }
